@@ -20,7 +20,6 @@
 CanDo view components.
 """
 
-from urllib import urlencode
 import pytz
 from xml.sax.saxutils import quoteattr
 
@@ -41,12 +40,14 @@ import zc.table.column
 from zc.catalog.interfaces import IExtentCatalog
 from zc.table.interfaces import ISortableColumn
 
+from z3c.rml import rml2pdf
 from z3c.form import form, field, button
 
 from schooltool.app.interfaces import ISchoolToolApplication
 from schooltool.app.interfaces import IApplicationPreferences
 from schooltool.app.catalog import buildQueryString
 from schooltool.course.interfaces import ISection
+from schooltool.course.interfaces import ISectionContainer
 from schooltool.common.inlinept import InheritTemplate
 from schooltool.common.inlinept import InlineViewPageTemplate
 from schooltool.gradebook.browser.gradebook import FlourishGradebookOverview
@@ -67,8 +68,11 @@ from schooltool.gradebook.browser.worksheet import FlourishWorksheetEditView
 from schooltool.gradebook.browser.pdf_views import FlourishGradebookPDFView
 from schooltool.gradebook.browser.pdf_views import WorksheetGrid
 from schooltool.person.interfaces import IPerson
-from schooltool.report.browser.report import RequestReportDownloadDialog
-from schooltool.report.report import ReportLinkViewlet
+from schooltool.report.browser.report import RequestRemoteReportDialog
+from schooltool.report.report import ReportTask
+from schooltool.schoolyear.interfaces import ISchoolYearContainer
+from schooltool.report.browser.report import FileArchiver
+from schooltool.report.browser.report import ReportArchivePage
 from schooltool.requirement.scoresystem import ScoreValidationError
 from schooltool.requirement.scoresystem import UNSCORED
 from schooltool.term.interfaces import ITerm, IDateManager
@@ -1723,13 +1727,6 @@ class CanDoGradeStudentValidateScoreView(FlourishGradebookValidateScoreView):
         return result
 
 
-class RequestStudentCompetencyReportView(RequestReportDownloadDialog):
-
-    def nextURL(self):
-        return '%s/student_competency_report.pdf' % absoluteURL(self.context,
-                                                                self.request)
-
-
 class StudentCompetencyReportPDFView(flourish.report.PlainPDFPage,
                                      StudentCompetencyRecordView):
 
@@ -1760,6 +1757,45 @@ class StudentCompetencyReportPDFView(flourish.report.PlainPDFPage,
             _('Course: ${courses}', mapping={'courses': courses}),
             ]
 
+    @property
+    def base_filename(self):
+        return 'student_skill_report_%s_%s_%s' % (
+            self.student.last_name,
+            self.student.first_name,
+            self.student.username)
+
+
+class CanDoStudentGradebookReportTask(ReportTask):
+
+    @property
+    def context(self):
+        int_ids = getUtility(IIntIds)
+        student = int_ids.queryObject(self.student_intid)
+        worksheet = int_ids.queryObject(self.worksheet_intid)
+        gradebook = ISkillsGradebook(worksheet)
+        student_gradebook = getMultiAdapter((student, gradebook),
+                                            ICanDoStudentGradebook)
+        student_gradebook.__parent__ = gradebook
+        return student_gradebook
+
+    @context.setter
+    def context(self, value):
+        student_gradebook = proxy.removeSecurityProxy(value)
+        student = student_gradebook.student
+        gradebook = student_gradebook.gradebook
+        worksheet = gradebook.context
+        int_ids = getUtility(IIntIds)
+        student_intid = int_ids.getId(student)
+        worksheet_intid = int_ids.getId(worksheet)
+        self.student_intid = student_intid
+        self.worksheet_intid = worksheet_intid
+
+
+class RequestStudentCompetencyReportView(RequestRemoteReportDialog):
+
+    report_builder = StudentCompetencyReportPDFView
+    task_factory = CanDoStudentGradebookReportTask
+
 
 class StudentCompetencyReportSkillsTablePart(table.pdf.RMLTablePart):
 
@@ -1806,46 +1842,6 @@ def getScoreSystems(gradebook):
     return result
 
 
-class RequestCompetencyCertificateView(RequestReportDownloadDialog):
-
-    template = ViewPageTemplateFile(
-        'templates/request_competency_certificate.pt')
-
-    @Lazy
-    def scoresystems(self):
-        result = getScoreSystems(self.context)
-        return result
-
-    def sorted_scoresystems(self):
-        collator = ICollator(self.request.locale)
-        return sorted(self.scoresystems.values(),
-                      key=lambda v: collator.key(v['obj'].title))
-
-    def is_selected(self, ss_info, score_info):
-        requested_passing_score = self.request.get(ss_info['name'])
-        if requested_passing_score is not None:
-            return requested_passing_score == score_info['label']
-        return score_info['value'] == ss_info['passing_score']
-
-    def nextURL(self):
-        url = '%s/competency_certificate.pdf' % absoluteURL(self.context,
-                                                            self.request)
-        params = {}
-        for ss_info in self.scoresystems.values():
-            requested_passing_score = self.request.get(ss_info['name'])
-            if requested_passing_score is not None:
-                params[ss_info['name']] = requested_passing_score
-        if params:
-            url += '?%s' % urlencode(params)
-        return url
-
-    def update(self):
-        if 'DOWNLOAD' in self.request:
-            self.request.response.redirect(self.nextURL())
-            return
-        super(RequestCompetencyCertificateView, self).update()
-
-
 class CompetencyCertificatePDFView(flourish.report.PlainPDFPage,
                                    CanDoGradeStudentBase):
 
@@ -1892,6 +1888,47 @@ class CompetencyCertificatePDFView(flourish.report.PlainPDFPage,
                 result[ss_info['name']] = ss_info['passing_score']
         return result
 
+    @property
+    def base_filename(self):
+        return 'certificate_of_competency_%s_%s_%s' % (
+            self.student.last_name,
+            self.student.first_name,
+            self.student.username)
+
+
+class RequestCompetencyCertificateView(RequestRemoteReportDialog):
+
+    report_builder = CompetencyCertificatePDFView
+    task_factory = CanDoStudentGradebookReportTask
+
+    template = ViewPageTemplateFile(
+        'templates/request_competency_certificate.pt')
+
+    @Lazy
+    def scoresystems(self):
+        result = getScoreSystems(self.context)
+        return result
+
+    def sorted_scoresystems(self):
+        collator = ICollator(self.request.locale)
+        return sorted(self.scoresystems.values(),
+                      key=lambda v: collator.key(v['obj'].title))
+
+    def is_selected(self, ss_info, score_info):
+        requested_passing_score = self.request.get(ss_info['name'])
+        if requested_passing_score is not None:
+            return requested_passing_score == score_info['label']
+        return score_info['value'] == ss_info['passing_score']
+
+    def schedule(self, task):
+        params = {}
+        for ss_info in self.scoresystems.values():
+            requested_passing_score = self.request.get(ss_info['name'])
+            if requested_passing_score is not None:
+                params[ss_info['name']] = requested_passing_score
+        task.request_params.update(params)
+        task.schedule(self.request)
+
 
 class CompetencyCertificateSkillsTablePart(table.pdf.RMLTablePart):
 
@@ -1938,11 +1975,22 @@ class CompetencyCertificateTableFilter(table.ajax.TableFilter):
         return ''
 
 
-class RequestStudentCompetencySectionReportView(RequestReportDownloadDialog):
+class SkillsGradebookReportTask(ReportTask):
 
-    def nextURL(self):
-        return '%s/student_competency_section_report.pdf' % (
-            absoluteURL(self.context, self.request))
+    @property
+    def context(self):
+        int_ids = getUtility(IIntIds)
+        worksheet = int_ids.queryObject(self.worksheet_intid)
+        gradebook = ISkillsGradebook(worksheet)
+        return gradebook
+
+    @context.setter
+    def context(self, value):
+        gradebook = proxy.removeSecurityProxy(value)
+        worksheet = gradebook.context
+        int_ids = getUtility(IIntIds)
+        worksheet_intid = int_ids.getId(worksheet)
+        self.worksheet_intid = worksheet_intid
 
 
 class StudentCompetencySectionReportPDFView(flourish.report.PlainPDFPage,
@@ -1957,6 +2005,19 @@ class StudentCompetencySectionReportPDFView(flourish.report.PlainPDFPage,
         schoolyear = ISchoolYear(term)
         return '%s | %s' % (term.title, schoolyear.title)
 
+    @property
+    def base_filename(self):
+        gradebook = proxy.removeSecurityProxy(self.context)
+        section = gradebook.section
+        courses = [c.__name__ for c in section.courses]
+        return 'aggregated_student_skill_reports_%s' % '_'.join(courses)
+
+
+class RequestStudentCompetencySectionReportView(RequestRemoteReportDialog):
+
+    report_builder = StudentCompetencySectionReportPDFView
+    task_factory = SkillsGradebookReportTask
+
 
 class NoHeaderPlainPageTemplate(flourish.report.PlainPageTemplate):
 
@@ -1984,6 +2045,7 @@ class StudentCompetencySectionReportPDFStory(flourish.report.PDFStory):
                 (student_gradebook, self.request, self.view, self),
                 name="student_scr")
             student_scr.update()
+            student_scr.__name__ = 'student_scr'
             result.append(student_scr.render())
         return ''.join(result)
 
@@ -2009,33 +2071,112 @@ class StudentSCRPart(flourish.report.PDFPart):
         return courses
 
     def scr_table(self):
+        self.view.__name__ = 'foobar'
         view = getMultiAdapter(
             (self.context, self.request),
             name='student_competency_report.html')
+        view.__name__ = 'student_competency_report.html'
         grades_table = getMultiAdapter(
             (self.context, self.request, view, self.manager),
             name="student_grades_table")
+        grades_table.__name__ = 'student_grades_table'
         # XXX: make StudentCompetencyReportSkillsTablePart registration
         #      more generic and get rid of this awful monkey patching
         rml_table = getMultiAdapter(
             (self.context, self.request, view, grades_table),
             table.interfaces.IRMLTable)
+        rml_table.__name__ = 'rml_table'
         rml_table.getColumnWidths = lambda x: '7% 10% 53% 15% 15%'
         rml_table.update()
         return rml_table.render()
 
 
-class SkillsGradebookReportLink(ReportLinkViewlet):
+class StudentCompetencyArchivePage(ReportArchivePage):
+    message_title = _("skill gradebooks archive")
+    base_filename = 'skill_gradebooks'
 
-    @property
-    def report_link(self):
-        skills = ISectionSkills(self.context)
-        if skills:
-            skillset = skills.values()[0]
-            return '%s/gradebook/%s' % (absoluteURL(skillset, self.request),
-                                        self.link)
 
-    def render(self, *args, **kw):
-        if not self.report_link:
-            return ''
-        return super(SkillsGradebookReportLink, self).render(*args, **kw)
+class ArchiveCompetencySectionReports(FileArchiver):
+
+    sections_by_term = None
+
+    report_builder = 'student_competency_section_report.pdf'
+
+    def collectTerms(self, schoolyear_id=None):
+        app = ISchoolToolApplication(None)
+        syc = ISchoolYearContainer(app)
+        archive_terms = []
+        for year_id, year in syc.items():
+            if (schoolyear_id is None or
+                year_id == schoolyear_id):
+                for term in year.values():
+                    archive_terms.append(term)
+        return archive_terms
+
+    def collectSections(self, terms):
+        archive_sections = []
+        if not terms:
+            return archive_sections
+        for term in terms:
+            sections = []
+            sc = ISectionContainer(term)
+            for section in sc.values():
+                sections.append(section)
+            if sections:
+                archive_sections.append((term, sections))
+        return archive_sections
+
+    def updateTargets(self):
+        terms = self.collectTerms()
+        sections = self.collectSections(terms)
+        self.sections_by_term = sections
+
+    def update(self):
+        FileArchiver.update(self)
+        self.updateTargets()
+
+    def renderReport(self, renderer, filename, archive):
+        renderer.update()
+        rml = renderer.render()
+        pdf_filename = renderer.filename
+        pdf_stream = rml2pdf.parseString(rml, filename=pdf_filename or None)
+        data = pdf_stream.getvalue()
+        archive.writestr(filename, data)
+
+    def renderSection(self, section, filename, archive):
+        request = self.request
+        if hasattr(request, 'clone'):
+            request = request.clone()
+        skills = ISectionSkills(section)
+        if not skills:
+            return
+        worksheet = skills.values()[0]
+        gradebook = ISkillsGradebook(worksheet)
+        renderer = self.queryView(
+            gradebook, request, self.report_builder)
+        if renderer is None:
+            return
+        self.renderReport(renderer, filename, archive)
+
+    def getFullFilename(self, year, term, section):
+        filename = '%s/%s/%s-skills.pdf' % (
+            year.__name__, term.__name__, section.__name__)
+        return filename
+
+    def render(self, archive):
+        total_files = sum([len(sections) for term, sections in self.sections_by_term])
+        file_n = 0
+        first_run = True
+        for term, sections in self.sections_by_term:
+            year = ISchoolYear(term)
+            self.setTitle(u'%s, %s' % (year.title, term.title))
+            for section in sections:
+                filename = self.getFullFilename(year, term, section)
+                self.progress(filename, file_n, total_files)
+                if first_run:
+                    # Push out update progress immediately
+                    self.view.task_progress.force()
+                first_run = False
+                self.renderSection(section, filename, archive)
+                file_n += 1
+        self.finish()
